#include <iostream>   // cout
#include <iomanip> 
#include <vector> 
#include <fstream> 
#include <stdexcept>
#include <stdlib.h> 

#include "strategy.h" 
#include "my_strategy.h"



// cviceni 
#include "alcaponestg.h"
#include "hip_strategy.h"
#include "kovarik.h"
#include "KyptaStrategy.h"
#include "langhammer.h"
#include "lessner_ulrych_3_strategy.h"
#include "ludek_cigler_strategy.h"
#include "ondra_k_str.h"
#include "PilatStrategy.h"
#include "vavrp4am_Strategy.h"
#include "evelina_strategy.h"

// domaci 
#include "dm_strategy.h"
#include "jr_strategy.h"
#include "moraj2am_strategy.h"
#include "pavel_ceska_strategy.h"
#include "Radek_strategy.h"


using namespace std; 

#define DRAND()	  (((double) rand()) / ( RAND_MAX + 1.0))


#define TRIALS 100
#define PRINT_TRIALS 30  

int rewards[2][2] = { { CC, CD }, { DC, DD } }; 

#define BASIC 11 // number of strategies 
#define CVIC 11 
#define DOM 5

int N = BASIC+CVIC+DOM;

string stratNames[BASIC+CVIC+DOM] = {
  
  "RandomStrategy", 
  "AlwaysCooperate", 
  "AlwaysDeceive",
  "TFT_(CopyPartner)", 
  "Alternate", 
  "Pavlov",
  // dalsi strategie 
  "Spite",
  "CCD",
  "DDC",
  "FQuick",
  "TF2T",
  // cviceni 
  "AlCapone",
  "Hip",
  "Revenger",
  "Kypta",
  "Langhammer",
  "JesteLepsiStrategie",
  "Ludek_Cigler",
  "Ondra1",
  "PilatStrategie",
  "vavrp4am_Strategy",
  "EvelinaStrategy",
  // domaci 
  "DM",
  "JrStrategy",
  "moraj2amSTRG",
  "Pavel_Ceska_Strategy",
  "RadekStrategy"
  };


Strategy* createStrategy(const string& name){
  
  if (name == "RandomStrategy") return new RandomStrategy(); 
  if (name == "AlwaysCooperate") return new AlwaysCooperate(); 
  if (name == "AlwaysDeceive") return new AlwaysDeceive();
  if (name == "TFT_(CopyPartner)") return new CopyPartner();
  if (name == "Alternate") return new Alternate();
  if (name == "Pavlov") return new Pavlov();

  if (name == "Spite") return new Spite(); 
  if (name == "CCD") return new CCD(); 
  if (name == "DDC") return new DDC(); 
  if (name == "FQuick") return new FQuick(); 
  if (name == "TF2T") return new TF2T();

  if (name == "AlCapone") return new AlCaponeStrategy();
  if (name == "Hip") return new HipStrategy();
  if (name == "Revenger") return new Revenger();
  if (name == "Kypta") return new KyptaStrategy();
  if (name == "Langhammer") return new Langhammer();
  if (name == "JesteLepsiStrategie") return new JesteLepsiStrategie();
  if (name == "Ludek_Cigler") return new LudekCiglerStrategy();
  if (name == "Ondra1") return new Ondra1();
  if (name == "PilatStrategie") return new PilatStrategy();
  if (name == "vavrp4am_Strategy") return new vavrp4am_Strategy();
  if (name == "EvelinaStrategy") return new EvelinaStrategy();


  if (name == "DM") return new DM_Strategy();
  if (name == "JrStrategy") return new JrStrategy();
  if (name == "moraj2amSTRG") return new MORAJ2AMSTRG();
  if (name == "Pavel_Ceska_Strategy") return new MyStrategy_pavel_ceska();
  if (name == "RadekStrategy") return new RadekStrategy();

  throw std::runtime_error("unknown strategy: "+name);
  return 0;
} 

class result {
public:
  int first_player;
  int second_player; 
};  

result compete(Strategy& strat1, Strategy& strat2) {

  int res1, res2;
  int rec1[TRIALS];
  int rec2[TRIALS]; 

  cout << strat1.getName() << " vs. " << strat2.getName() << endl; 

  strat1.reset(); strat2.reset(); // prepare for competition 

  result res; 
  res.first_player = 0;
  res.second_player = 0; 

  // compete 
  for (int i=0; i<TRIALS; i++) {
   
    // ask for decision 
    rec1[i] = res1 = strat1(); 
    rec2[i] = res2 = strat2();
    

    // some strategies returns illegal value 
    if ((res1 != COOPERATE) && (res1 != DECEIVE)) {
      if (DRAND()>0.5)
	rec1[i] = res1 = COOPERATE;
      else
	rec1[i] = res1 = DECEIVE;
    } 

    if ((res2 != COOPERATE) && (res2 != DECEIVE)) {
      if (DRAND()>0.5)
	rec2[i] = res2 = COOPERATE;
      else
	rec2[i] = res2 = DECEIVE;
    } 


    // rewards
    strat1.reward( rewards[res1][res2] ); 
    strat2.reward( rewards[res2][res1] ); 
  
    res.first_player += rewards[res1][res2]; 
    res.second_player += rewards[res2][res1]; 
  }

  cout << strat1.getName() << " vs. " << strat2.getName() << endl; 
  cout <<"Player1:  ";
  for (int i=0; i<PRINT_TRIALS; i++) cout << ((rec1[i]) ? "D" : "C");
  cout << " => " << res.first_player << endl; 
  cout << "Player2:  "; 
  for (int i=0; i<PRINT_TRIALS; i++) cout << ((rec2[i]) ? "D" : "C"); 
  cout << " => " << res.second_player << endl << endl; 

  
  return res; 
} 


int main() {

  srand( time(0) );

  // array of strategies 
  vector<Strategy*> strats; 
  strats.resize(N);


  for (int i=0; i<N; i++)
    strats[i] = createStrategy( stratNames[i] ); 

  int results[N][N]; 
  int sum_results[N]; 


    for (int i=0; i<N; i++) sum_results[i] = 0; 

    result res; 

    // compete 
    for (int i=0; i< N; i++)  
      for (int j=0; j<i; j++) {
	res = compete( *strats[i], *strats[j] ); 
	
	results[i][j] = res.first_player;
	results[j][i] = res.second_player;
	
	sum_results[i] += results[i][j]; 
	sum_results[j] += results[j][i]; 
      } 
  
    // compete with myself
    for (int i=0; i<N; i++) {
      Strategy* myCopy = createStrategy( strats[i]->getName() );
    
      res = compete( *strats[i], *myCopy); 
      
      results[i][i] = (res.first_player + res.second_player)/2;
      
      sum_results[i] += results[i][i]; 

      delete myCopy;
    } 
    
  /*
  // print results
  for (int i=0; i<N; i++) {
    for (int j=0; j<N; j++) {
      cout <<  results[i][j] << " "; 
    } 
    cout << endl; 
  } 
  */

  for (int i=0; i<N; i++) 
    cout << setw(30) << strats[i]->getName() << ":  " << sum_results[i]/N << endl; 
  
  ofstream fout("results.txt");
  for (int i=0; i<N; i++) 
    fout << sum_results[i]/N << " " << strats[i]->getName() << endl; 
  
  fout.close();

  for (unsigned i=0; i<strats.size(); i++) 
    delete strats[i]; 

  return 0; 
} 
