#ifndef MY_STRATEGY_H
#define MY_STRATEGY_H

#include "strategy.h"

class Spite : public Strategy {
public:
  Spite()  { reset(); } 
  virtual int operator()() { return response; } 
  virtual void reset() { response = COOPERATE; }
  virtual void reward(int _reward) { if (response == COOPERATE && _reward == CD) response = DECEIVE; }

  virtual string getName() { return "Spite"; } 
private:
  int response; 
}; 

class CCD : public Strategy {
public:
  CCD() { reset(); } 
  virtual void reset() { counter = 0; }
  virtual string getName() { return "CCD"; }

  virtual int operator()() { 
    counter %= 3; 
    switch (counter) {
    case 0:
      counter++;
      return COOPERATE;
      break;
    case 1:
      counter++;
      return COOPERATE;
      break;
    case 2:
      counter++;
      return DECEIVE;
      break; 
    }
    return COOPERATE; // should not be reached
  }
private: 
  int counter; 
}; 

class DDC : public Strategy {
public:
  DDC() { reset(); } 
  virtual void reset() { counter = 0; }
  virtual string getName() { return "DDC"; }

  virtual int operator()() { 
    counter %= 3; 
    switch (counter) {
    case 0:
      counter++;
      return DECEIVE;
      break;
    case 1:
      counter++;      
      return DECEIVE;
      break;
    case 2:
      counter++;
      return COOPERATE;
      break; 
    }
    return COOPERATE; // should not be reached
  }
private: 
  int counter; 
}; 


// always cooperate
// but if oponent deceived, deceive  in the next turn, but then cooperate  
class FQuick : public Strategy {
public:
  FQuick() { reset(); }
  virtual void reset() { deceive = false; }
  virtual string getName() { return "FQuick"; } 
  
  virtual int operator()() {

    if (deceive) return DECEIVE;
    else return COOPERATE; 
  } 

  virtual void reward(int _reward) { 
    if (_reward == CD) deceive = true;  // I cooperated, oponent deceived
    else deceive = false;
  } 
  
private:
  bool deceive;  
};  

// copy partner - called TFT "Tit for Tat"
// TF2T - "Tit for 2 tats" 
// start CC
//  if oponent DD then D else S  
class TF2T : public Strategy {
public:
  TF2T() { reset(); } 
  virtual void reset() { deceived = 0; }
  virtual string getName() { return "TF2T"; }
  
  virtual void reward(int _reward) { 
    if (_reward == CD || _reward == DD) 
      deceived++;
    else 
      deceived = 0; 
  }
  
  virtual int operator()() {
    if (deceived >= 2) 
      return DECEIVE;
    else 
      return COOPERATE; 
  }
  

private:
  int deceived; 
}; 


#endif 
