#ifndef STRATEGY_H
#define STRATEGY_H

#include <stdlib.h> 
#include <string>

#define Me_C   0 
#define Me_D   1
#define Oponent_C 0 
#define Oponent_D 2 

#define COOPERATE 0
#define DECEIVE  1 

#define CC 3
#define CD 0
#define DC 5
#define DD 1

using namespace std;

#define DRAND()	  (((double) rand()) / ( RAND_MAX + 1.0))

class Strategy {
public:
  virtual ~Strategy() {};
  virtual void reset() {};                 // zresetovani strategie, vola se na zacatku souboje
  virtual int operator()() = 0;            // souboj, vraci COOPERATE nebo DECEIVE
  virtual void reward(int _reward) {};     // informace o vysledku soubouje (CC/CD/DC/DD)
  virtual string getName() = 0; 
};



class RandomStrategy : public Strategy {
public:
  virtual int operator()() {  if (DRAND()>0.5) return COOPERATE; else return DECEIVE;  } 
  virtual string getName() { return "RandomStrategy"; }  
};  

class AlwaysCooperate : public Strategy {
public:
  virtual int operator()() { return COOPERATE; } 
  virtual string getName() { return "AlwaysCooperate"; }  
}; 

class AlwaysDeceive : public Strategy {
public:
  virtual int operator()() { return DECEIVE; }
  virtual string getName() { return "AlwaysDeceive"; }   
}; 

class CopyPartner : public Strategy {
public:
  
  CopyPartner()  { reset(); }; 

  virtual int operator()() { return last; }
 
  virtual void reward(int _reward) {
    if (_reward == CC || _reward == DC) last = COOPERATE;
    else last = DECEIVE;
  } 

  virtual void reset() { last = COOPERATE; }

  virtual string getName() { return "TFT_(CopyPartner)"; }  

private:
  int last;
}; 


class Alternate : public Strategy {
public:
  Alternate()  { reset(); }
  virtual int operator()() { last = !last; return last; } 
  virtual void reset() { last = DECEIVE; }
  virtual string getName() { return "Alternate"; }  
private:
  int last; 
};  


class Pavlov : public Strategy {
public:
  Pavlov()  { reset(); } 
  int operator()() { return response; } 
  virtual void reset() { response = COOPERATE; } 
  virtual void reward(int _reward) { if (_reward == CD) response = DECEIVE; 
  if (_reward == DC)  response = COOPERATE; } 
  virtual string getName() { return "Pavlov"; }  
private:
  int response; 
}; 



#endif
